function pp_common_plots(PPEXP,ID,RMB,color1,color2)

%PP_COMMON_PLOTS Standart plots for rtg use (rtg 2.x).
%   PPEXP   Name of the calling pp_xxx function, e.g. 'pp_tau0_20'
%   ID      Plot ID number. This is the 'MODE' in rtg, defined in MODE
%           menu xxx_xdef.m.
%   RMB     Data channel number (or vector). For rtg, this is defined
%           in FREQCH menu in xxx_xdef.m.
%   COLOR1  Primary color for each data channels. E.g. {'m','y','m','y'}
%   COLOR2  Secondary color for each data channel.
%
%   The following IDs are currently supported:
%
%   -------------------------------------------------------------------
%   ID      Mnemonic    Description
%   -------------------------------------------------------------------
%   1       s0,b0       Raw signal and raw background.
%   2       s(-b)       Signal lagprofile, background subtracted
%                       for longpulses.
%   3       s0          Raw signal lagprofile.         (pp_spp) 
%   4       b           Signal's associated background (pp_bpp)
%   5       b,c,T       Signal's associated background, calibration and
%                       system temperature.
%   6       tx          Transmission envelope, real and imagenary part.
%   7       abs tx      Transmission, magnitude of the complex envilope.
%   8       pha tx      Transmission, phase of the complex envilope Also
%                       the effective doppler shift corresponding to
%                       phase droop.
%   9       tx/tx0      Transmission, real and imagenary part, normalized
%                       to be unity at the point where abs(tx) is
%                       mazimum.
%   10      ACF0        ACF, without bgr subtr, without range scaling.
%   11      SP0         Spectra, without bgr subtr, without range scaling.
%   12      ACF(-b)     ACF, bgr subracted for long pulses, always
%                       scaled by 1/r^2 of the gates.
%   13      SP(-b)      Spectra, bgr subracted for longpulses, 
%                       always scaled by 1/r^2 of the gates.
%
%   Note 1. When adding IDs, use numbers < 100. Do not modify existing
%           ID numbers. It is perfectly legal to use decimal numbers if
%           desired.
%
%   Note 2. We require here that our calling function (PPEXP) supports a
%           specific interface which in effect allows us to call certain
%           subfunctions of the PPEXP function here. In particular, the
%           PPEXP function must support the following two ways of
%           calling it:
%
%               freq  = PPEXP('call','freq',rmb,type)
%               noise = PPEXP('call','mean_noise',rmb)
%
%           See slso the subfunctions freq() and mean_noise() at the end
%           of this file. 
%           See pp_tau0_20.m for an example of such a PPEXP function.
%
%   Based on several versions of various pp_xxx's by Jm and Am.
%
%   Last modified 28-Nov-1999 Jm.


%   MODIFICATIONS
%   -------------   
%   19-Nov-1999 Jm: Original
%   28-Nov-1999 Jm: PPEXP mediated to greq() and mean_noise() via global
%                   rtg_CONST, instead as a parameter. This ensures that the
%                   plot function code can be copied without modifications
%                   to the pp_xxx routines for testing, modifications, etc.
%    1-Dec-1999 Jm: Axis handling changed to work via pp_axis.m  
%    7-Dec-1999 Jm: ACF/SP computations now handle gating
%   16-Dec-1999 Jm: Replaced axes(h) by set(fig,'currentax',h).

%{----------------------------------------------------------------------


global CALTEMP ELEV GATE GUP 
global rtg_CONST; rtg_CONST.ppexp = PPEXP;


if ID < 10 
    % single axis on most plots
    use_singleax; 
end


if ID == 1 
%======================================================================
% s0,b0 : Raw signal and background
%======================================================================
%{
    if pp_ismulti(RMB) 
        pperr('Multiple channels not allowed (now)'); 
        return
    end

    s = pp_spp(RMB);
    [w,tail] = pp_wpp(RMB);
    bgr = pp_bpp(RMB);
    bgr = bgr(1:length(s));
    r   = pp_ppr(RMB); 
    xlbl = 'Range';
    if ppopt_altitude
        r = range2alt(r,ELEV);
        xlbl = 'Altitude';
    end        
    plot(...
        r,bgr, color2{RMB},...
        r,s,color1{RMB}, ...
        r(tail),s(tail),[color1{RMB} '--'] ...
	 );
    
    lag_lbl = pp_laglbl(pp_lix2lag(ppopt('lix'),RMB));
    pp_annot(RMB, freq(RMB,'s'), xlbl, [lag_lbl ', lag 0'], '', 0);	 
%}s0,b0


elseif ID == 2 
%======================================================================
% s : Signal lagprofile, with options applied
%======================================================================
%{
    w_lbl = ''; r2_lbl = ''; n_lbl = ''; xlbl = 'Range';
    lix = ppopt('lix');
    oldhold = ishold;
    gating = ppopt_gating(RMB);
    gt_lbl = [' <' num2str(gating) '>'];
    for rmb = RMB;
        z = pp_spp(rmb);
        if pp_islp(rmb) & ( lix == 1 )
            z = z - pp_bgr(rmb);
        end
        if ppopt_deweight
            [w,tail] = pp_wpp(rmb);
            z = z ./ w;
            w_lbl = ' / w';
        end            
        r = pp_ppr(rmb);
        if ppopt_rangescaling
            z = z .* pp_scaler(rmb, r);
            r2_lbl = '  \cdot r^2';
        end
        if ppopt_normalize
            z = z / mean_noise(rmb);
            n_lbl = ' / c';
        end                        
        if ppopt_altitude
            r = range2alt(r,ELEV);
            xlbl = 'Altitude';
        end
        if gating > 1
            r = rtg_gater(r,gating);
            z = rtg_gater(z,gating);
        end
        plot( r, z, color1{rmb})
        hold on
    end    
    if ~oldhold, hold off, end

    lag_lbl = pp_laglbl(pp_lix2lag(lix,RMB));
    pp_annot( RMB, freq(RMB,'s'), xlbl, ...
            [lag_lbl gt_lbl r2_lbl w_lbl n_lbl], '', 0);
%}s


elseif ID == 3
%======================================================================
% s0 : Signal lagprofile, options NOT applied
%======================================================================
%{
    oldhold = ishold;
    for rmb = RMB;
        r = pp_ppr(rmb);
        if ppopt_altitude
            r = range2alt(r,ELEV);
            xlbl = 'Altitude [km]';
        else        
            xlbl = 'Range [km]';
        end
        s = pp_spp(rmb);      
        plot( r, s, color1{rmb})
        hold on
    end    
    if ~oldhold, hold off, end

    lag_lbl = pp_laglbl(pp_lix2lag(ppopt('lix'),RMB));
    pp_annot( RMB, freq(RMB,'s'), xlbl, lag_lbl, '', 0);
%}s0


elseif ID == 4
%======================================================================
% b : Signal's associated background
%======================================================================
%{
    oldhold = ishold; n_lbl = ''; w_lbl = '';
    for rmb = RMB
        b1 = pp_bpp(rmb);
        b = GUP.pp - 1;    
        if ppopt_normalize
            n = mean_noise(rmb);
            b1 = b1 / n;
            n_lbl = ' / c';
        end 
        if ppopt_deweight & xdef_gkill(rmb)
            b1 = b1 / 2;
            w_lbl = ' / w';
        end            

        t = b * pp_samplinginterval('ms');       
        plot( t, b1, color1{rmb});
        hold on
    end
    if ~oldhold, hold off, end
    pp_annot(RMB, freq(RMB,'b'), 'time [ms]',['lag 0' n_lbl w_lbl], '', 0);
    grid
%}b

elseif ID == 5
%==========================================================================
% b,c :  Signal's associated background, calibration and system temperature
%==========================================================================
%{
    oldhold = ishold; n_lbl = ''; w_lbl = ''; systemp = [];
    for rmb = RMB
        pp = GUP.pp(1:end);
        cpp = GUP.cpp(1:end);
        bgr = pp_bpp(rmb);
        cal = pp_cpp(rmb);
        n = length(cal);
        c = pp(end)-n + cpp;
        if ppopt_normalize
            n = mean_noise(rmb);
            bgr = bgr / n;
            cal = cal / n;
            n_lbl = ' / c';
        end 
        plot (...
	     pp-1, bgr, color1{rmb}, ...
	        c, cal, [color1{rmb} 'o'] ...
	     )
        hold on
        ave_b = mean(bgr(end-100:end));
        ave_c = mean(cal) - ave_b;
        systemp = [systemp round(( ave_b / ave_c ) * CALTEMP)];
    end
    if ~oldhold, hold off, end

    T_str = sprintf('%dK ', systemp);
    pp_annot(RMB, freq(RMB,'b'), 'Address', ['Lag 0' n_lbl], ...
                sprintf('T=%s (%dK)',T_str,CALTEMP), 0);
%}b,c


elseif ID == 6
%==========================================================================
% tx :  Transmission, real and imagenary part
%==========================================================================
%{
    oldhold = ishold;
    for rmb = RMB;
        [z,t] = pp_tx(rmb);
        if isempty(z)
            pperr('TX samples not available')
            return;
        end            
        x = real(z);
        y = imag(z);
        a = 1:GUP.txsamples;
	plot (...
	     t(a),x(a), [color1{rmb} '-o'], ...
	     t(a),y(a), color2{rmb} ...
	     )
        hold on
    end    
    if ~oldhold, hold off, end
    pp_annot(RMB, freq(RMB,'t'), 'Time [ \mus ]', 'Re & Im', '',0);
%}tx


elseif ID == 7
%==========================================================================
% abs tx :  Transmission, magnitude of the complex envilope
%==========================================================================
%{
    oldhold = ishold;
    for rmb = RMB;
        [z,t] = pp_tx(rmb);
        a = 1:GUP.txsamples;
        plot (t(a), abs(z(a)), [color1{rmb} '.-'])
        hold on
    end    
    if ~oldhold, hold off, end
    grid
    pp_annot(RMB, freq(RMB,'t'), 'Time [\mus]', 'Magnitude','',0);  
%}abs tx


elseif ID == 8
%==========================================================================
% pha tx :  Transmission, phase of the complex envilope
%           Also the effective doppler shift corresponding to phase droop.
%==========================================================================

    oldhold = ishold;
    a = 1:GUP.txsamples;
    for rmb = RMB;
        [z,t] = pp_tx(rmb);
        pha = angle(z) * 180/pi;
        [vel,frq,flip] = tx_sample_fit(t(a),pha(a));
	plot ( t(a), pha(a), [color1{rmb}, '-o'] );
        hold on
    end    
    if ~oldhold, hold off, end
    grid
    pp_annot(RMB, freq(RMB,'t'), 'Time [\mus]', 'Phase []', ...
            sprintf(' D=%1.1f m/s  \\Delta\\Phi=%d', vel, round(flip)), 0);
%}pha tx


elseif ID == 9
%==========================================================================
% tx/tx0:  Transmission, real and imagenary part, normalized to be unity
%          at the point where abs(tx) is mazimum. 
%==========================================================================
%{
    oldhold = ishold;
    for rmb = RMB;
        [z,t] = pp_tx(rmb);
        if isempty(z)
            pperr('TX samples not available')
            return;
        end  
        k = find(abs(z) > 0);
        if ~isempty(k)
            z = z ./ z(k(1));
        end
        x = real(z);
        y = imag(z);
        a = 1:GUP.txsamples;
	plot (...
	     t(a),x(a), [color1{rmb} '-o'], ...
	     t(a),y(a), color2{rmb} ...
             )
        hold on
    end
    if ~oldhold, hold off, end
    grid
    pp_annot(RMB, freq(RMB,'t'), 'Time [ \mus ]', 'Re & Im', '',0);
%}tx


elseif ID == 10 | ID == 11 | ID == 12 | ID == 13
%==========================================================================
% Signal autocorrelation functions and spectra, with and without 
% background subtraction.
%
% ID 10 : ACF 
% ID 11 : SP  
% ID 12 : ACF with backbground subtraction and r^2 scaling
% ID 13 : SP with background subtraction and r^2 scaling
%
%==========================================================================
%{
    if pp_ismulti(RMB) 
        pperr('Multiple channels not allowed (now)');
        return
    end
    rmb = RMB;
    
    %----------------------------------------------------------------
    % Compute acf range gates to a matrix, with columns as the gates
    %----------------------------------------------------------------

    [gates,gating,range] = pp_getgate(rmb);

    if pp_isac(rmb)
        ACF = ac_acfmatrix(gates, GUP.ssa(rmb), GUP.ACnbits,...
                    GUP.AClags(end), GUP.Nfract, GUP.Nsamples, GUP.Lagweight);
    else
        subtr = (ID == 12 | ID == 13);
        ACF = lp_acfmatrix( rmb, gates, subtr );
    end

    if gating > 1
        ACF = (rtg_gater((ACF)',gating))';
    end        

    ngates = size(ACF,2);

    %-------------------------------
    % Normalize by noise injection :
    %-------------------------------

    if ppopt_normalize
        snoise = mean_noise(rmb);
        ACF = ACF ./ snoise;
    end        

    %----------------
    % Divide by r^2 :
    %----------------

    if ( ID == 12 | ID == 13 ) & ppopt_rangescaling
        ACF = ACF .* kron(ones(size(ACF,1),1), (range .^ 2) / 100^2);
        wtype = 'r^2';
    else
        wtype = '';    
    end        

    %----------
    % Unbiase :
    %----------

    if pp_isac(rmb)
        maxlag = GUP.AClags(end);  
    else
        maxlag = GUP.LPlags(end);
    end                

    if ppopt_biasedacf
        ACF = ACF / (maxlag+1);
        wtype = [wtype 'b'];
    else
        weight = 1 ./ ( (maxlag+1) - (0:maxlag)' );
        ACF = ACF .* kron( ones(1,ngates), weight);
        wtype = [wtype 'u'];
    end        

    %------------
    % Plot ACFs :
    %------------

    nlags = size(ACF,1);
    lag = (0:nlags-1)*GUP.Tau*1e6;
    c = {color1{rmb},color2{rmb}};

    if ID == 10 | ID == 12 % acf mode

        type = ppopt('acfplottype');

        if ngates == 1 | strcmp(type,'line')
            use_singleax
            pp_acf_lineplot(ACF,lag,range,{color1{rmb},color2{rmb}});
            pp_zoomcheck('auto');

        else
            if strcmp(type,'waterfall')
                h = pp_axis(3,'create');
                ax(h(1));  rtg_waterf(lag, real(ACF), c, 'xy');
                ax(h(2));  rtg_waterf(lag, imag(ACF), c, 'xy');

            elseif strcmp(type,'mesh')
                h = pp_axis(2,'create');
                ax(h(1));  pp_acf_meshplot(real(ACF), range, lag);
                ax(h(2));  rtg_waterf(lag, real(ACF), c{1});

            elseif strcmp(type,'pcolor')		
                h = pp_axis(2,'create');
		ax(h(1));  pp_acf_pcolor(real(ACF), range, lag);
                ax(h(2));  rtg_waterf(lag, real(ACF), c{1});
	
            elseif strcmp(type,'image')
                h = pp_axis(2,'create');
                ax(h(1));  pp_imageplot(real(ACF'), range, lag, 'Lag [\mus]');     
                ax(h(2));  rtg_waterf(lag, real(ACF), c{1});

            elseif strcmp(type,'contour')
                h = pp_axis(2,'create');
                ax(h(1));  pp_contourplot(real(ACF'), range, lag, 'Lag [\mus]');
                ax(h(2));  rtg_waterf(lag, real(ACF), c{1});
            end
            
        end           
    end %}

    %---------------------------
    % Compute and plot spectra :
    %---------------------------

    if ID == 11 | ID == 13  % spec mode
         
        flimit = ppopt('f');
	if flimit
	    fmin = flimit(1);
            fmax = flimit(2);
        else
            fmin = -Inf;
            fmax = +Inf;
        end

        [spe,frq] = acfspec( ACF, 0, GUP.Fs, fmin, fmax, ppopt_windowedsp,0,0);
        frq = frq/1000; %freq in kHz

        type = ppopt('specplottype');
        c = {color1{rmb},color2{rmb}};

        if ngates == 1 | strcmp(type,'line')
            use_singleax
            pp_sp_lineplot(frq,spe,range,c{1});

        elseif strcmp(type,'waterfall')
            h = pp_axis(3,'create');
            c = {color1{rmb},color2{rmb}};
            ax(h(1));  rtg_waterf( frq, real(spe), c, 'xy', range);
            ax(h(2));  rtg_waterf( lag, real(ACF), c, 'xy',range);

        elseif strcmp(type,'pcolor')		
            h = pp_axis(2,'create');
            ax(h(1));  pp_acf_pcolor( real(spe), range, frq);
            ax(h(2));  rtg_waterf( frq, real(spe), c{1});

        elseif strcmp(type,'mesh')
            h = pp_axis(2,'create');
            ax(h(1));  pp_sp_meshplot( real(spe), range, frq, '[kHz]');
            ax(h(2));  rtg_waterf( frq, real(spe), c{1});

        elseif strcmp(type,'image')
            h = pp_axis(2,'create');
            ax(h(1));  pp_imageplot( real(spe'), range, frq, '[kHz]');
            ax(h(2));  rtg_waterf( frq, real(spe), c{1});

        elseif strcmp(type,'contour')
            h = pp_axis(2,'create');
            ax(h(1));  pp_contourplot( real(spe'), range, frq, '[kHz]');
            ax(h(2));  rtg_waterf( frq, real(spe), c{1});
        end
        
        if ppopt_windowedsp
            wtype = [wtype,'H'];
        else    
            wtype = [wtype,'n'];
        end
    end

    % Draw header

    pp_acfsp_header(rmb, gates, range, wtype, gating);


else %}{--------------------- [ else ]------------------------------------

    pperr('Illegal MODE');
    
end %}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Interface to a few experiment specific functions which are actually
% implemented as subfunctions of the pp_xxx functions. The pp_exp function
% then is assumed to be the caller of the pp_common_plots function, and its
% name must be available as the parameter rtg_CONST.ppexp here.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function f = freq(rmb,type)
%------------------------------------------------------------------------------
% Return frequency of a given data channel, of given type ('s', 'b' or  't').
%------------------------------------------------------------------------------
%{
    global rtg_CONST
    f = feval(rtg_CONST.ppexp, 'call', 'freq', rmb,type);
    return    
%}freq


function noise = mean_noise(rmb)
%------------------------------------------------------------------------
% Return mean, background subtracted, calibration noise value for the
% given res mem channel 'rmb'. This value is used in normalizing signal
% and background data. (GUISDAP calls this 'scaling by noise injection').
%
% (The problem in using simply <cpp> - <bpp> is that we may need to
% truncate the "bgr" curves more or less heavily, and might need to skipp
% points of cpp also, in an experiment specific way.)
%-------------------------------------------------------------------------
%{
    global rtg_CONST
    noise = feval(rtg_CONST.ppexp, 'call', 'mean_noise', rmb);
    return
%}mean_noise    


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Local subfunctions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function use_singleax
%--------------------
    global rtg_FIGhandle
    h = pp_axis(1,'create');
    set(rtg_FIGhandle(2),'CurrentAxes',h);


function ax(h)
%-------------
    global rtg_FIGhandle
    set(rtg_FIGhandle(2),'CurrentAxes',h);
