function pp_setgate(rmb,varargin)

%PP_SETGATE Set gating for rtg ACF and SP computations.
%
%   PP_SETGATE(Inf,'default')       ==> default on all RMB
%   PP_SETGATE(RMB,RANGELIST)
%   PP_SETGATE(RMB,RANGELIST,GATING)
%   PP_SETGATE(RMB,RANGE1,RANGE2,GATING)
%
%   Note that all expect the first of the call forms implicitly
%   require experiment specifications to be available in the global
%   variable GUP.
%
%   Note also that for alternating codes, only the 'fully weighted' gates
%   are accepted -- lower gates are silently ignored. 
% 
%   Examples
%
%       pp_setgate(1,300,400,1)
%         ==>
%           Each availabe raw range gate between 300 and 400 km range.
%           In other words, no gating.
%
%       pp_setgate(1,300,400,3)
%         ==>
%           Let the available raw range gates between 300 and 400 km be
%           x1,x2,x3,... xN. Then the output gates yk are
%               y1 = mean(x1,x2,x3) 
%               y2 = mean(x4,x5,x6) 
%               etc ...
%
%       pp_setgate(1,300:10:400,3)
%         ==>
%           Range gates roughly centered to ranges 300,310,320,... 400
%           km. Each output gate is mean of three input gates. 
%                              
%       pp_setgate(1,[300 330 500],5) 
%         ==>
%           Range gates roughly centered to ranges 300, 330 and 500
%           km. Each output gate is mean of 5 input gates.
%                                        
%   See also pp_sethate.m for setting gates based on gate hights
%   and pp_getgate to see the actual center points of the resulting gates.
%
%   -------
%
%   Note for developpers. This routine works in close collaboration with
%   rtg ACF/SP computation routines. The "output gates" given in the
%   above examples refer to the gating as _after_ the ACF/SP computation.
%   They do not refer (say) to the contents of the GATE(n).gates variable
%   after the execution of this function! Instead, use pp_getgate to see the
%   center point and gating factor of the output gates.
%
%   See also: pp_getgate
%
%   7-Dec-1999 Jm
%   21-May-2000 Jm (this header expanded).

%{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if nargin == 2
        if ischar(varargin{1})
            set_default_gating(rmb,varargin{1});
        else
            R = varargin{1};            % Vector of gate ranges
            Gating = 1;
            set_anyform_gating(rmb,R,Gating);
        end                        

    elseif nargin == 3

        R = varargin{1};            % Vector of gate ranges
        Gating = round(varargin{2});       % Gating factor
        set_anyform_gating(rmb,R,Gating);

    elseif nargin == 4

        R1 = varargin{1};           % Range of first gate
        R2 = varargin{2};           % Range of last gate
        Gating = round(varargin{3});       % Gating factor
        set_uniform_gating(rmb,R1,R2,Gating);

    else
        error('Wrong number of input arguments')
    end 
                          
%}pp_setgate


function set_default_gating(rmb,command)
%---------------------------------------
% Set flags so that when gating info is requested by pp_getgate(),
% the default gating is returned.
%
% Note that we cannot set the actual gating here, because it is
% allowed to call this function BEFORE the necessary experiment
% related information is available.
%{
    global GATE
    if ischar(command)
        if rmb == Inf & strcmp(command,'default')
            GATE = 1;
        else
            error('Invalid parameters')
        end                
    else
        error('Invalid parameters')
    end                            
%}set_default_gating


function set_uniform_gating(RMB,R1,R2,Gating)
%--------------------------------------------
% Generate all gates from g(R1) to g(R2). From these gates the ACF/SP
% computations then form the final gates by taking mean of the first
% Gating gates, next Gating gates, and so on.
%{

    global GATE GUP

    for rmb = RMB
        r0 = pp_gaterange(1,rmb);   % range of first fully weighted range gate
        dr = GUP.dr;                % gate separation

        if pp_isac(rmb)
            nfract = GUP.Nfract;
            nbits = GUP.ACnbits;
            nsamples = GUP.Nsamples;
            maxgate = (nsamples-1) - (nbits-1)*nfract - 2*(nfract-1);
        else
            maxgate = GUP.Nsamples - GUP.LPlags(end);
        end

        g1 = floor((R1-r0)/dr) + 1;
        if g1 < 1, g1 = 1; end

        g2 = floor((R2-r0)/dr) + 1;
        if g2 > maxgate
            g2 = maxgate;
        end        

        ngates = g2-g1+1;
        ngates = ngates - rem(ngates,Gating);
        g2 = g1 + ngates - 1;

        if isnumeric(GATE)
            GATE = [];
        end
        GATE(rmb).gates = g1:g2;
        GATE(rmb).gating = Gating;
        GATE(rmb).r = [R1,R2];
        GATE(rmb).mode = 'uniform';
    end
    
%}set_uniform_gating


function set_anyform_gating(RMB,R,Gating)
%---------------------------------------
% Generate gates in groups if Gating consecutive gates, centered at the
% given ranges. The ACF/SP computations will then take means within
% those gate groups to form the final data. 

    global GATE GUP

    for rmb = RMB

        r0 = pp_gaterange(1,rmb);   % range of first fully weighted range gate
        dr = GUP.dr;                % gate separation

        if pp_isac(rmb)
            nfract = GUP.Nfract;
            nbits = GUP.ACnbits;
            nsamples = GUP.Nsamples;
            maxgate = (nsamples-1) - (nbits-1)*nfract - 2*(nfract-1);
        else
            maxgate = GUP.Nsamples - GUP.LPlags(end);
        end

        % First gate in each group of gates

        g1 = round( (R-r0)/dr + 1 - (Gating - 1)/2 ); 

        % Ensure that the gate groups fit fully into the available gates

        K = find( g1 < 1 | g1 > floor(maxgate - Gating) );
        if ~isempty(K)
            g1(K) = [];
        end        

        % Form the gates, something like (with length(R) = 3, Gating = 4):
        %   
        %     g1 = [ 3          13              30          ]  ==>
        %  gates = [ 3 4 5 6    13 14 15 16     30 31 32 33 ]   

        a = kron(g1,ones(1,Gating));    
        % = [ 3 3 3 3   13 13 13 13   30 30 30 30 ]

        b = kron(ones(size(g1)),0:(Gating-1));
        % = [ 0 1 2 3   0  1  2  3    0  1  2  3  ]

        if isnumeric(GATE)
            GATE = [];
        end
        GATE(rmb).gates = a + b;
        GATE(rmb).gating = Gating;      % save gating factor
        GATE(rmb).r = R;                % save the range specification
        GATE(rmb).mode = 'anyform';     % save specification mode

    end
    
%}set_anyform_gating    
