function value = ppopt(Name,Value)

% Function interface to pp_generic options (pp_OPT)
%
% PPOPT                display all options and their values
% PPOPT ?              display list of (abreviated) option names          
% VAL = PPOPT(NAME)    return value of the given option
% PPOPT(NAME,VALUE)    set value of the given option
%
% The NAME can be appreviated as long as it unambiguous.
% The character case is not significant. The allowed NAMEs
% are defined in pp_OPT, see subfunction dbase below.
%
% 3-Dec-1999 Jm

    global pp_OPT
    
    if nargin > 2 
        error('Wrong number of input parameters')
    end

    if isempty(pp_OPT)
        dbase;
    end        

    if nargin == 0
        disp_all;
        return
    end

    if nargin == 1 & Name == '?'
        % Return list option names, first few letters
        L = 6;            
        value = pp_OPT.name{1}(1:min(L,end));
        for n = 2:length(pp_OPT.name)
            value = [value ' ' pp_OPT.name{n}(1:min(L,end))];
        end
        return                
    end        

    % Find the official name corresponding the given name

    K = matcher(Name, pp_OPT.name);
    if isempty(K)
        error(['Illegal parameter (' Name ')']);
    elseif length(K) > 1
        mlist = ':';
        for n = 1:length(K)
            mlist = [mlist ' ' pp_OPT.name{K(n)}];
        end            
        error(['Ambiguous parameter (' Name mlist ')']);
    end           
    
    officialname = pp_OPT.name{K};
    internalname = pp_OPT.Name{K};
    
    if nargin == 1

        % GET the current value for parameter Name -- just the field in pp_OPT
    
        value = getfield(pp_OPT,internalname);
        
    elseif nargin == 2
    
        % We are going to SET a value. For the non-numeric parameters, we
        % accept abbreviations, and so need to complete Value first.
        
        if pp_OPT.typ(K) == 0 | pp_OPT.typ(K) == 3  % non-numeric parameter
           
            possiblevalues = pp_OPT.val{K};
            n = matcher(Value,possiblevalues);
            if isempty(n)
                error(['Illegal parameter value (' Value ')']);
            elseif length(n) > 1
                error(['Ambiguous parameter value (' Value ')']);
            else
                Value = possiblevalues{n};
            end            
            
        else
            % Value is known to be numeric, so force that interpretation
            % even when the input might be given as a string
            if ischar(Value)
                Value = str2num(Value);
            end
            % Special handling of fminfmax:
            if strcmp(officialname,'fminfmax')
                if length(Value) == 1
                    Value = [-abs(Value) +abs(Value)];
                end
            end                                    
        end
        
        % Assign the Value to the appropriate field of pp_OPT                            
        
        pp_OPT = setfield(pp_OPT,pp_OPT.Name{K},Value);
    
        % return the Value, as a confirmation
        
        value = Value;
                            
    end                            

%}pp_opt


function disp_all
%----------------
%{
    global pp_OPT
    for k = 1:length(pp_OPT.Name)
        value = getfield(pp_OPT,pp_OPT.Name{k});
        if isnumeric(value)
            fprintf(1,'%14s:  [',pp_OPT.name{k});
            fprintf(1,' %g',value);
            fprintf(1,' ]\n');
        else
            fprintf(1,'%14s:  %s\n',pp_OPT.name{k}, value);
        end
    end
%}                                


function dbase
%-------------
%{
    global pp_OPT

    dbase = cell2struct( {
    %    internal name   type     possible values
    %-----------------------------------------------------------------------    
        'acfplottype',    3,     {'contour';'image';'line'; ...
                                  'mesh';'pcolor';'waterfall'};
        'altid',          0,     {'no';'yes'};
        'biased_ACF',     0,     {'no';'yes'};
        'deweight',       0,     {'no';'yes'};
        'fminfmax',       2,     {};
        'gatingfactor',   1,     {'no';'yes'};
        'gray_images',    0,     {'no';'yes'};
        'lix',            1,     {};
        'normalize',      0,     {'no','yes'};
        'range_scaling',  0,     {'no';'yes'};
        'specplottype',   3,     {'mesh';'pcolor';'contour';...
                                  'image';'line';'waterfall'};
        'windowed_SP',    0,     {'no';'yes'}
    %------------------------------------------------------------------------
    },{ 'name',          'typ',  'val'},       2 );

    for k=1:length(dbase)
        Name{k}  = dbase(k).name;
        val{k}   = dbase(k).val;
        typ(k)   = dbase(k).typ;
    end        
    
    pp_OPT = [];

    % hardcode defaults       

    pp_OPT.acfplottype      = 'mesh';
    pp_OPT.altid            = 'no';
    pp_OPT.biased_ACF       = 'no';
    pp_OPT.deweight         = 'no';
    pp_OPT.fminfmax         = [-inf inf];
    pp_OPT.gatingfactor     = 0;
    pp_OPT.gray_images      = 'no';
    pp_OPT.lix              = 1;
    pp_OPT.normalize        = 'no';
    pp_OPT.range_scaling    = 'no';
    pp_OPT.specplottype     = 'image';
    pp_OPT.windowed_SP      = 'no';

    % the following 'options' that are not considered customizable

    pp_OPT.gatecolor        = {'y','c','m','g','w'};	   

    pp_OPT.gain_scaling     = 'no';
    pp_OPT.specmaplen       = 64;
    pp_OPT.acfcmaplen       = 64;

    % add IO control info

    pp_OPT.Name             = Name;
    pp_OPT.name             = lowercase(Name);
    pp_OPT.val              = val;
    pp_OPT.typ              = typ;
    
%}dbase    

